"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlgoIndexerTransaction = void 0;
const types_1 = require("../../types");
const algoUtils_1 = require("../../utils/algoUtils");
const constants_1 = require("../../utils/constants");
const managed_1 = require("../../utils/managed");
const utils_1 = require("../../utils/utils");
const TransactionBase_1 = require("../TransactionBase");
const web3 = require("web3");
let AlgoIndexerTransaction = class AlgoIndexerTransaction extends TransactionBase_1.TransactionBase {
    get txid() {
        return this.hash;
    }
    get stdTxid() {
        return (0, algoUtils_1.txIdToHexNo0x)(this.txid);
    }
    get hash() {
        if (this.data.transaction.id === undefined) {
            return "";
        }
        return this.data.transaction.id;
    }
    get reference() {
        return [(0, algoUtils_1.base64ToHex)(this.data.transaction.note || "")];
    }
    get stdPaymentReference() {
        let paymentReference = this.reference.length === 1 ? (0, utils_1.prefix0x)(this.reference[0]) : "";
        try {
            // try to parse out
            paymentReference = (0, utils_1.prefix0x)(web3.utils.hexToString((0, utils_1.prefix0x)(paymentReference)));
        }
        catch (e) {
            return utils_1.ZERO_BYTES_32;
        }
        if (!(0, utils_1.isValidBytes32Hex)(paymentReference)) {
            paymentReference = utils_1.ZERO_BYTES_32;
        }
        return paymentReference;
    }
    get unixTimestamp() {
        if (this.data.transaction.roundTime === undefined) {
            return 0;
        }
        return this.data.transaction.roundTime;
    }
    get sourceAddresses() {
        return [this.data.transaction.sender];
    }
    get receivingAddresses() {
        if (this.type === "pay" && this.data.transaction.paymentTransaction) {
            return [this.data.transaction.paymentTransaction.receiver];
        }
        else if (this.type === "axfer" && this.data.transaction.assetTransferTransaction) {
            return [this.data.transaction.assetTransferTransaction.receiver];
        }
        // TODO check other transaction types
        return [];
    }
    get fee() {
        return (0, utils_1.toBN)(this.data.transaction.fee);
    }
    get spentAmounts() {
        if (this.data.transaction.txType === "pay" && this.data.transaction.paymentTransaction) {
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee.add((0, utils_1.toBN)(this.data.transaction.paymentTransaction.amount)),
                },
            ];
        }
        else {
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee,
                },
            ];
        }
    }
    get receivedAmounts() {
        if (this.type === "pay" && this.data.transaction.paymentTransaction) {
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(this.data.transaction.paymentTransaction.amount),
                },
            ];
        }
        else if (this.type === "axfer" && this.data.transaction.assetTransferTransaction) {
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(this.data.transaction.assetTransferTransaction.amount),
                },
            ];
        }
        return [];
    }
    get type() {
        return this.data.transaction.txType;
    }
    get isNativePayment() {
        return this.type === "pay";
    }
    get currencyName() {
        if (this.type === "pay") {
            return constants_1.ALGO_NATIVE_TOKEN_NAME;
        }
        else if (this.type === "axfer" && this.data.transaction.assetTransferTransaction) {
            return this.data.transaction.assetTransferTransaction.assetId.toString();
        }
        return "";
    }
    get elementaryUnits() {
        return (0, utils_1.toBN)(constants_1.ALGO_MDU);
    }
    get successStatus() {
        // TODO research this further
        return types_1.TransactionSuccessStatus.SUCCESS;
    }
    async paymentSummary(client, inUtxo, utxo, makeFullPayment) {
        if (!this.isNativePayment) {
            return { isNativePayment: false };
        }
        return {
            isNativePayment: true,
            sourceAddress: this.sourceAddresses[0],
            receivingAddress: this.receivingAddresses[0],
            spentAmount: this.spentAmounts[0].amount,
            receivedAmount: this.receivedAmounts[0].amount,
            paymentReference: this.stdPaymentReference,
            oneToOne: true,
            isFull: true,
        };
    }
};
AlgoIndexerTransaction = __decorate([
    (0, managed_1.Managed)()
], AlgoIndexerTransaction);
exports.AlgoIndexerTransaction = AlgoIndexerTransaction;
//# sourceMappingURL=AlgoIndexerTransaction.js.map