"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UtxoTransaction = void 0;
const __1 = require("../..");
const types_1 = require("../../types");
const constants_1 = require("../../utils/constants");
const utxoUtils_1 = require("../../utils/utxoUtils");
const TransactionBase_1 = require("../TransactionBase");
const managed_1 = require("../../utils/managed");
// Transaction types and their description
// - coinbase        : transaction that mints new coins
// - payment         : what you get from node
// - partial_payment : transaction with some vout of vins added to additional data
// - full_payment    : transaction with vouts for all vins added to additional data
let UtxoTransaction = class UtxoTransaction extends TransactionBase_1.TransactionBase {
    constructor(data, additionalData) {
        super(data, additionalData);
        this.data.vout.forEach((vout) => {
            this.processOutput(vout);
        });
        this.synchronizeAdditionalData();
        this.assertAdditionalData();
    }
    get txid() {
        return this.data.txid;
    }
    get stdTxid() {
        return (0, __1.unPrefix0x)(this.txid);
    }
    get hash() {
        return this.data.hash;
    }
    get reference() {
        let references = [];
        for (let vo of this.data.vout) {
            if (vo.scriptPubKey.hex.substring(0, 2) == (0, __1.unPrefix0x)((0, __1.toHex)(utxoUtils_1.WordToOpcode.OP_RETURN))) {
                let dataSplit = vo.scriptPubKey.asm.split(" ");
                references.push(dataSplit[1]);
            }
        }
        return references;
    }
    get stdPaymentReference() {
        let paymentReference = this.reference.length === 1 ? (0, __1.prefix0x)(this.reference[0] || "") : "";
        if (!(0, __1.isValidBytes32Hex)(paymentReference)) {
            paymentReference = __1.ZERO_BYTES_32;
        }
        return paymentReference;
    }
    get unixTimestamp() {
        return this.data.blocktime;
    }
    get sourceAddresses() {
        if (this.type === "coinbase") {
            // Coinbase transactions mint coins
            return [undefined];
        }
        return this.additionalData.vinouts.map((mapper) => { var _a; return (_a = mapper === null || mapper === void 0 ? void 0 : mapper.vinvout) === null || _a === void 0 ? void 0 : _a.scriptPubKey.address; });
    }
    get receivingAddresses() {
        var _a;
        /* istanbul ignore if */
        // Transaction should always have vout array
        if (!((_a = this.data) === null || _a === void 0 ? void 0 : _a.vout)) {
            return [];
        }
        return this.data.vout.map((vout) => vout.scriptPubKey.address);
    }
    get fee() {
        const reducerFunctionVinOuts = (prev, vout) => { var _a; return prev.add((0, __1.toBN)(Math.round((((_a = vout === null || vout === void 0 ? void 0 : vout.vinvout) === null || _a === void 0 ? void 0 : _a.value) || 0) * constants_1.BTC_MDU).toFixed(0))); };
        const reducerFunctionVouts = (prev, vout) => prev.add((0, __1.toBN)(Math.round(vout.value * constants_1.BTC_MDU).toFixed(0)));
        if (this.type === "full_payment") {
            if (this.additionalData && this.additionalData.vinouts) {
                let inSum = this.additionalData.vinouts.reduce(reducerFunctionVinOuts, (0, __1.toBN)(0));
                let outSum = this.data.vout.reduce(reducerFunctionVouts, (0, __1.toBN)(0));
                return inSum.sub(outSum);
            }
        }
        if (this.type === "coinbase") {
            // Coinbase transactions mint coins
            return (0, __1.toBN)(0);
        }
        throw (0, __1.MccError)("fee can't be calculated for `payment` and `partial_payment` transaction types");
    }
    get spentAmounts() {
        if (this.type === "coinbase") {
            // Coinbase transactions mint coins
            return [
                {
                    amount: (0, __1.toBN)(0),
                },
            ];
        }
        return this.additionalData.vinouts.map((mapper) => {
            var _a, _b, _c, _d;
            return {
                address: (_b = (_a = mapper === null || mapper === void 0 ? void 0 : mapper.vinvout) === null || _a === void 0 ? void 0 : _a.scriptPubKey) === null || _b === void 0 ? void 0 : _b.address,
                amount: (0, __1.toBN)(Math.round((((_c = mapper === null || mapper === void 0 ? void 0 : mapper.vinvout) === null || _c === void 0 ? void 0 : _c.value) || 0) * constants_1.BTC_MDU).toFixed(0)),
                utxo: (_d = mapper === null || mapper === void 0 ? void 0 : mapper.vinvout) === null || _d === void 0 ? void 0 : _d.n,
            };
        });
    }
    get receivedAmounts() {
        return this.data.vout.map((vout) => {
            return {
                address: vout.scriptPubKey.address,
                amount: (0, __1.toBN)(Math.round((vout.value || 0) * constants_1.BTC_MDU).toFixed(0)),
                utxo: vout.n,
            };
        });
    }
    get type() {
        if (this.data.vin.length === 0 || this.data.vin[0].coinbase) {
            return "coinbase";
        }
        let hasUndefined = false;
        let hasDefined = false;
        for (let i = 0; i < this.additionalData.vinouts.length; i++) {
            let vinOut = this.additionalData.vinouts[i];
            if (vinOut === undefined) {
                hasUndefined = true;
            }
            else {
                hasDefined = true;
            }
        }
        if (hasDefined && !hasUndefined) {
            return "full_payment";
        }
        if (hasDefined) {
            return "partial_payment";
        }
        return "payment";
    }
    get isNativePayment() {
        // On these chains there are no other types of transactions
        return true;
    }
    get currencyName() {
        /* istanbul ignore next */
        // This must be shadowed
        throw new Error("Method must be implemented in different sub class");
    }
    get elementaryUnits() {
        return (0, __1.toBN)(constants_1.BTC_MDU);
    }
    get successStatus() {
        return types_1.TransactionSuccessStatus.SUCCESS;
    }
    async paymentSummary(client, vinIndex, voutIndex, makeFullPayment) {
        this.assertValidVinIndex(vinIndex, true);
        this.assertValidVoutIndex(voutIndex, true);
        // Refresh the inputs if needed
        if (makeFullPayment) {
            await this.makeFullPayment(client);
        }
        else {
            // If vinIndex is not defined or null, then we do not need the input
            if (vinIndex != null) {
                await this.vinVoutAt(vinIndex, client);
            }
        }
        let sourceAddress = vinIndex != null ? this.sourceAddresses[vinIndex] : undefined;
        let receivingAddress = voutIndex != null ? this.receivingAddresses[voutIndex] : undefined;
        let oneToOne = !!sourceAddress && !!receivingAddress;
        let isFull = this.type === "full_payment";
        if (isFull) {
            let inFunds = (0, __1.toBN)(0);
            let returnFunds = (0, __1.toBN)(0);
            let outFunds = (0, __1.toBN)(0);
            let inFundsOfReceivingAddress = (0, __1.toBN)(0);
            for (let vinAmount of this.spentAmounts) {
                if (sourceAddress && vinAmount.address === sourceAddress) {
                    inFunds = inFunds.add(vinAmount.amount);
                }
                if (receivingAddress && vinAmount.address === receivingAddress) {
                    inFundsOfReceivingAddress = inFundsOfReceivingAddress.add(vinAmount.amount);
                }
                if (oneToOne && vinAmount.address != sourceAddress && vinAmount.address != receivingAddress) {
                    oneToOne = false;
                }
            }
            for (let voutAmount of this.receivedAmounts) {
                if (sourceAddress && voutAmount.address === sourceAddress) {
                    returnFunds = returnFunds.add(voutAmount.amount);
                }
                if (receivingAddress && voutAmount.address === receivingAddress) {
                    outFunds = outFunds.add(voutAmount.amount);
                }
                // Outputs without address do not break one-to-one condition
                if (oneToOne && !voutAmount.address && voutAmount.amount.gt((0, __1.toBN)(0))) {
                    oneToOne = false;
                }
                if (oneToOne && voutAmount.address && voutAmount.address != sourceAddress && voutAmount.address != receivingAddress) {
                    oneToOne = false;
                }
            }
            return {
                isNativePayment: true,
                sourceAddress,
                receivingAddress,
                spentAmount: inFunds.sub(returnFunds),
                receivedAmount: outFunds.sub(inFundsOfReceivingAddress),
                paymentReference: this.stdPaymentReference,
                oneToOne,
                isFull,
            };
        }
        let spentAmount = sourceAddress ? this.spentAmounts[vinIndex].amount : (0, __1.toBN)(0);
        let receivedAmount = receivingAddress ? this.receivedAmounts[voutIndex].amount : (0, __1.toBN)(0);
        oneToOne = false;
        return {
            isNativePayment: true,
            sourceAddress,
            receivingAddress,
            spentAmount,
            receivedAmount,
            paymentReference: this.stdPaymentReference,
            oneToOne,
            isFull,
        };
    }
    ///////////////////////////////
    //// Utxo specific methods ////
    ///////////////////////////////
    /**
     * Asserts whether the vin index is in valid range. If not, exception is thrown.
     * @param vinIndex vin index
     */
    assertValidVinIndex(vinIndex, canBeNull = false) {
        if (canBeNull && vinIndex == null) {
            return;
        }
        if (vinIndex == null || vinIndex < 0 || vinIndex >= this.sourceAddresses.length) {
            throw new Error("Invalid vin index");
        }
    }
    /**
     * Asserts whether the vout index is in valid range. If not, exception is thrown.
     * @param voutIndex vout index
     */
    assertValidVoutIndex(voutIndex, canBeNull = false) {
        if (canBeNull && voutIndex == null) {
            return;
        }
        if (voutIndex == null || voutIndex < 0 || voutIndex >= this.receivingAddresses.length) {
            throw new Error("Invalid vout index");
        }
    }
    /**
     * Asserts whether vinvouts Mapper is full mapper and only has unique and matching indices
     */
    assertAdditionalData() {
        var _a, _b;
        if (this.additionalData) {
            if (((_a = this.additionalData.vinouts) === null || _a === void 0 ? void 0 : _a.length) !== this.data.vin.length) {
                throw new Error("Bad format, Additional data vinvouts and data vin length mismatch");
            }
            (_b = this.additionalData) === null || _b === void 0 ? void 0 : _b.vinouts.forEach((vinvout, ind) => {
                if (vinvout && vinvout.index !== ind) {
                    throw new Error("Additional data corrupted: indices mismatch");
                }
            });
        }
    }
    synchronizeAdditionalData() {
        let tempAdditionalData = this.additionalData;
        this.additionalData = {
            vinouts: new Array(this.data.vin.length).fill(undefined),
        };
        if (tempAdditionalData === null || tempAdditionalData === void 0 ? void 0 : tempAdditionalData.vinouts) {
            for (let vinvout of tempAdditionalData.vinouts) {
                if (vinvout) {
                    if (vinvout.index < 0 || vinvout.index >= this.data.vin.length) {
                        throw new Error("vinvout wrong index out of range");
                    }
                    this.additionalData.vinouts[vinvout.index] = vinvout;
                    this.processOutput(vinvout.vinvout);
                }
            }
        }
    }
    /**
     * Extract vout on vout index. If vout index is not valid or data is corrupted, exception is thrown.
     * @param voutIndex vout index
     * @returns vout on vout index
     */
    extractVoutAt(voutIndex) {
        this.assertValidVoutIndex(voutIndex);
        const toReturn = this.data.vout[voutIndex];
        if (toReturn.n !== voutIndex) {
            throw (0, __1.MccError)(`Vin and vout transaction miss match; requested index ${voutIndex}, found index ${toReturn.n}`);
        }
        return toReturn;
    }
    /**
     *
     * @param vinIndex vin index
     * @returns vin on vin index
     */
    extractVinAt(vinIndex) {
        this.assertValidVinIndex(vinIndex);
        return this.data.vin[vinIndex];
    }
    /**
     * Gets the vout corresponding to vin in the given vin index. If the vout is not fetched, the
     * input transaction is read and the corresponding vout is obtained
     * @param vinIndex vin index
     * @param client mcc client for fetching input the transactions
     * @returns input vout of the relevant utxo of the input transaction on the vin on the index `vinIndex`
     */
    async extractVinVoutAt(vinIndex, client) {
        const vinObject = this.extractVinAt(vinIndex);
        if (!vinObject.txid) {
            return {
                index: vinIndex,
                vinvout: undefined,
            };
        }
        const connectedTrans = await client.getTransaction(vinObject.txid);
        if (connectedTrans === null) {
            return {
                index: vinIndex,
                vinvout: undefined,
            };
        }
        return {
            index: vinIndex,
            vinvout: connectedTrans.extractVoutAt(vinObject.vout),
        };
    }
    /**
     * Gets the vout corresponding to vin in the given vin index. If it is not stored in the additionalData,
     * it's fetched and updated using the client.
     * @param vinIndex vin index
     * @param client mcc client to fetch transaction data
     * @returns input vout of the relevant utxo of the input transaction on the vin on the index `vinIndex`
     */
    async vinVoutAt(vinIndex, client) {
        // note: vinouts list is always initialized
        if (!this.additionalData) {
            this.additionalData = {
                vinouts: new Array(this.data.vin.length).fill(undefined),
            };
        }
        let vinVout = this.additionalData.vinouts[vinIndex];
        if (vinVout) {
            return vinVout;
        }
        if (!client) {
            throw new Error("MCC Client required.");
        }
        vinVout = await this.extractVinVoutAt(vinIndex, client);
        this.additionalData.vinouts[vinIndex] = vinVout;
        return vinVout;
    }
    /**
     * Checks if the input on index `vinIndex` is fetched.
     * @param vinIndex vin index
     * @returns true if input on the index `vinIndex` is already fetched.
     */
    isSyncedVinIndex(vinIndex) {
        this.assertValidVinIndex(vinIndex);
        return !!this.additionalData.vinouts[vinIndex];
    }
    /**
     * Fetches all the vin data from input transactions and makes the payment full_payment.
     * @param client mcc client to use for fetching input data
     */
    async makeFullPayment(client) {
        let promises = [];
        this.synchronizeAdditionalData();
        for (let i = 0; i < this.data.vin.length; i++) {
            promises.push(this.vinVoutAt(i, client));
        }
        await Promise.all(promises);
        // console.log(this.additionalData);
        // for (let i = 0; i < (this.additionalData?.vinouts?.length || 0); i++) {
        //    // console.log(this.additionalData?.vinouts![i]);
        // }
    }
    processOutput(vout) {
        if (!vout) {
            return;
        }
        if (vout.scriptPubKey.address) {
            return;
        }
        if (vout.scriptPubKey.addresses && vout.scriptPubKey.addresses.length === 1) {
            vout.scriptPubKey.address = vout.scriptPubKey.addresses[0];
        }
        // otherwise `address` stays undefined
    }
};
UtxoTransaction = __decorate([
    (0, managed_1.Managed)()
], UtxoTransaction);
exports.UtxoTransaction = UtxoTransaction;
//# sourceMappingURL=UtxoTransaction.js.map