"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../../src");
const chai_1 = require("chai");
const algoCreateConfig = {
    algod: {
        url: process.env.ALGO_ALGOD_URL || "",
        token: process.env.ALGO_ALGOD_TOKEN || "",
    },
    indexer: {
        url: process.env.ALGO_INDEXER_URL || "",
        token: process.env.ALGO_INDEXER_TOKEN || "",
    },
};
const hei = 21374440;
describe(`Algo transaction from algod block VS from indexer compare`, async () => {
    describe(`Compare transactions  ${hei}`, function () {
        let MccClient;
        let block;
        let BTrans;
        let ITrans;
        before(async function () {
            MccClient = new src_1.MCC.ALGO(algoCreateConfig);
            let tblock = await MccClient.getBlock(hei);
            if (tblock !== null) {
                block = tblock;
                const bttrans = tblock.transactions[0];
                if (bttrans !== null) {
                    BTrans = bttrans;
                    let titrans = await MccClient.getIndexerTransaction(BTrans.txid);
                    if (titrans !== null) {
                        ITrans = titrans;
                    }
                }
            }
        });
        it("Should compare transaction txid ", async function () {
            (0, chai_1.expect)(BTrans.txid).to.eq(ITrans.txid);
        });
        it("Should compare transaction hash ", async function () {
            (0, chai_1.expect)(BTrans.hash).to.eq(ITrans.hash);
        });
        it("Should compare transaction std txid ", async function () {
            (0, chai_1.expect)(BTrans.stdTxid).to.eq(ITrans.stdTxid);
        });
        it("Should compare transaction reference ", async function () {
            (0, chai_1.expect)(BTrans.reference.length).to.eq(ITrans.reference.length);
            ITrans.reference.sort();
            BTrans.reference.sort();
            for (let i = 0; i < BTrans.reference.length; i++) {
                (0, chai_1.expect)(BTrans.reference[i]).to.eq(ITrans.reference[i]);
            }
        });
        it("Should compare transaction stdPaymentReference ", async function () {
            (0, chai_1.expect)(BTrans.stdPaymentReference).to.eq(ITrans.stdPaymentReference);
        });
        it("Should compare transaction std txid ", async function () {
            (0, chai_1.expect)(BTrans.stdTxid).to.eq(ITrans.stdTxid);
        });
        it("Should compare transaction unix timestamp ", async function () {
            (0, chai_1.expect)(BTrans.unixTimestamp).to.eq(ITrans.unixTimestamp);
        });
        it("Should compare transaction sourceAddresses ", async function () {
            (0, chai_1.expect)(BTrans.sourceAddresses.length).to.eq(ITrans.sourceAddresses.length);
            ITrans.sourceAddresses.sort();
            BTrans.sourceAddresses.sort();
            for (let i = 0; i < BTrans.sourceAddresses.length; i++) {
                (0, chai_1.expect)(BTrans.sourceAddresses[i]).to.eq(ITrans.sourceAddresses[i]);
            }
        });
        it("Should compare transaction receivingAddresses ", async function () {
            (0, chai_1.expect)(BTrans.receivingAddresses.length).to.eq(ITrans.receivingAddresses.length);
            ITrans.receivingAddresses.sort();
            BTrans.receivingAddresses.sort();
            for (let i = 0; i < BTrans.receivingAddresses.length; i++) {
                (0, chai_1.expect)(BTrans.receivingAddresses[i]).to.eq(ITrans.receivingAddresses[i]);
            }
        });
        it("Should compare transaction fee ", async function () {
            (0, chai_1.expect)(BTrans.fee.toString()).to.eq(ITrans.fee.toString());
        });
        it("Should compare transaction spentAmounts ", async function () {
            (0, chai_1.expect)(BTrans.spentAmounts.length).to.eq(ITrans.spentAmounts.length);
            ITrans.spentAmounts.sort();
            BTrans.spentAmounts.sort();
            for (let i = 0; i < BTrans.spentAmounts.length; i++) {
                if (ITrans.spentAmounts[i].address && BTrans.spentAmounts[i].address) {
                    (0, chai_1.expect)(BTrans.spentAmounts[i].address).to.eq(ITrans.spentAmounts[i].address);
                }
                (0, chai_1.expect)(BTrans.spentAmounts[i].amount.toString()).to.eq(ITrans.spentAmounts[i].amount.toString());
            }
        });
        it("Should compare transaction receivedAmounts ", async function () {
            (0, chai_1.expect)(BTrans.receivedAmounts.length).to.eq(ITrans.receivedAmounts.length);
            ITrans.receivedAmounts.sort();
            BTrans.receivedAmounts.sort();
            for (let i = 0; i < BTrans.receivedAmounts.length; i++) {
                if (ITrans.receivedAmounts[i].address && BTrans.receivedAmounts[i].address) {
                    (0, chai_1.expect)(BTrans.receivedAmounts[i].address).to.eq(ITrans.receivedAmounts[i].address);
                }
                (0, chai_1.expect)(BTrans.receivedAmounts[i].amount.toString()).to.eq(ITrans.receivedAmounts[i].amount.toString());
            }
        });
        it("Should compare transaction type ", async function () {
            (0, chai_1.expect)(BTrans.type).to.eq(ITrans.type);
        });
        it("Should compare transaction isNativePayment ", async function () {
            (0, chai_1.expect)(BTrans.isNativePayment).to.eq(ITrans.isNativePayment);
        });
        it("Should compare transaction currencyName ", async function () {
            (0, chai_1.expect)(BTrans.currencyName).to.eq(ITrans.currencyName);
        });
        it("Should compare transaction elementaryUnits ", async function () {
            (0, chai_1.expect)(BTrans.elementaryUnits.toString()).to.eq(ITrans.elementaryUnits.toString());
        });
        it("Should compare transaction successStatus ", async function () {
            (0, chai_1.expect)(BTrans.successStatus).to.eq(ITrans.successStatus);
        });
        it("Should compare transaction payment summary ", async function () {
            const sum1 = await BTrans.paymentSummary();
            const sum2 = await ITrans.paymentSummary();
            for (let key in sum1) {
                (0, chai_1.expect)(sum1.hasOwnProperty(key)).to.eq(sum2.hasOwnProperty(key));
                // @ts-ignore
                (0, chai_1.expect)(sum1[key]).to.eq(sum2[key]);
            }
        });
        it("Should compare transaction data ", async function () { });
    });
});
//# sourceMappingURL=transactionCompare.test.js.map