"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
};
describe("Transaction Xrp tests ", function () {
    let MccClient;
    before(async function () {
        MccClient = new src_1.MCC.XRP(XRPMccConnection);
    });
    describe("Offer create transaction ", function () {
        let transaction;
        const txid = "C32ACF8CCF4F48B7AE097873AA2B7672DC66E05D4F1B3133DA90D1F476B1EAC6";
        before(async function () {
            transaction = await MccClient.getTransaction(txid);
        });
        it("Should find transaction ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            (0, chai_1.expect)(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            (0, chai_1.expect)(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            (0, chai_1.expect)(transaction.hash).to.eq(txid);
        });
        it("Should get transaction reference array ", async function () {
            (0, chai_1.expect)(transaction.reference.length).to.eq(1);
        });
        it("Should get standardized transaction reference ", async function () {
            (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1646397932);
        });
        it("Should get source address ", async function () {
            (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("rETx8GBiH6fxhTcfHM9fGeyShqxozyD3xe");
        });
        it("Should get receiving address ", async function () {
            (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(0);
        });
        it("Should get fee ", async function () {
            (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(20);
        });
        it("Should spend amount ", async function () {
            (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(1);
            (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("rETx8GBiH6fxhTcfHM9fGeyShqxozyD3xe");
            (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(20);
        });
        it("Should received amount ", async function () {
            (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(0);
        });
        it("Should get type ", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("OfferCreate");
        });
        it("Should check if native payment ", async function () {
            (0, chai_1.expect)(transaction.isNativePayment).to.eq(false);
        });
        it("Should get currency name ", async function () {
            (0, chai_1.expect)(transaction.currencyName).to.eq("");
        });
        it("Should get elementary unit ", async function () {
            (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(1000000);
        });
        it("Should get success status ", async function () {
            (0, chai_1.expect)(transaction.successStatus).to.eq(0);
        });
        it.skip("Should get payment summary ", async function () {
            const summary = await transaction.paymentSummary(MccClient);
            console.log(summary);
            if (summary.tokenElementaryUnits)
                console.log("tokenElementaryUnits: ", summary.tokenElementaryUnits.toString(10));
            if (summary.receivedTokenAmount)
                console.log("receivedTokenAmount: ", summary.receivedTokenAmount.toString(10));
            if (summary.spentAmount)
                console.log("spentAmount: ", summary.spentAmount.toString(10));
            if (summary.receivedAmount)
                console.log("receivedAmount: ", summary.receivedAmount.toString(10));
        });
    });
    describe("Token Transfer transaction ", function () {
        let transaction;
        const txid = "AB790D026B7C2E42113A1834A032CED756090D0FCE7EA995E0ACDE9660C88836";
        before(async function () {
            transaction = await MccClient.getTransaction(txid);
        });
        it("Should find transaction ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            (0, chai_1.expect)(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            (0, chai_1.expect)(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            (0, chai_1.expect)(transaction.hash).to.eq(txid);
        });
        it("Should get transaction reference array ", async function () {
            (0, chai_1.expect)(transaction.reference.length).to.eq(1);
        });
        it("Should get standardized transaction reference ", async function () {
            (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1646398292);
        });
        it("Should get source address ", async function () {
            (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("rBy7gEjA6AJytwZAUKYfXvGAf5Y1koFCX1");
        });
        it("Should get receiving address ", async function () {
            (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.receivingAddresses[0]).to.eq("rP9vJS39NRRK3ET1FpvWHFrtbXYESBYayf");
        });
        it("Should get fee ", async function () {
            (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(20);
        });
        it("Should spend amount ", async function () {
            (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(1);
            (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("rBy7gEjA6AJytwZAUKYfXvGAf5Y1koFCX1");
            (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(20);
        });
        it("Should received amount ", async function () {
            (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(0);
        });
        it("Should get type ", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("Payment");
        });
        it("Should check if native payment ", async function () {
            (0, chai_1.expect)(transaction.isNativePayment).to.eq(false);
        });
        it("Should get currency name ", async function () {
            (0, chai_1.expect)(transaction.currencyName).to.eq("CX1");
        });
        it("Should get elementary unit ", async function () {
            (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(1000000);
        });
        it("Should get success status ", async function () {
            (0, chai_1.expect)(transaction.successStatus).to.eq(0);
        });
        it("Should get payment summary ", async function () {
            var _a, _b, _c, _d;
            const summary = await transaction.paymentSummary(MccClient);
            (0, chai_1.expect)(summary.isNativePayment).to.eq(false);
            (0, chai_1.expect)(summary.sourceAddress).to.eq("rBy7gEjA6AJytwZAUKYfXvGAf5Y1koFCX1");
            (0, chai_1.expect)(summary.receivingAddress).to.eq("rP9vJS39NRRK3ET1FpvWHFrtbXYESBYayf");
            (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(20);
            (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(undefined);
            (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
            (0, chai_1.expect)((_c = summary.tokenElementaryUnits) === null || _c === void 0 ? void 0 : _c.toNumber()).to.eq(1);
            (0, chai_1.expect)((_d = summary.receivedTokenAmount) === null || _d === void 0 ? void 0 : _d.toNumber()).to.eq(10);
            (0, chai_1.expect)(summary.oneToOne).to.eq(true);
            (0, chai_1.expect)(summary.isFull).to.eq(true);
        });
    });
    describe("Transaction with native currency ", function () {
        let transaction;
        const txid = "75C7402BB60B574F7876627307EDACAA0A6830EB01692F150555999BDDBB4650";
        before(async function () {
            transaction = await MccClient.getTransaction(txid);
        });
        it("Should find transaction ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            (0, chai_1.expect)(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            (0, chai_1.expect)(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            (0, chai_1.expect)(transaction.hash).to.eq(txid);
        });
        it("Should get transaction reference array ", async function () {
            (0, chai_1.expect)(transaction.reference.length).to.eq(0);
        });
        it("Should get standardized transaction reference ", async function () {
            (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1647510771);
        });
        it("Should get source address ", async function () {
            (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("r3zUhJWabAMMLT5n631r2wDh9RP3dN1bRy");
        });
        it("Should get receiving address ", async function () {
            (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.receivingAddresses[0]).to.eq("rpE6gE8jEN1trDwQwe47VmgDL5y6m3XX2n");
        });
        it("Should get fee ", async function () {
            (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(45);
        });
        it("Should spend amount ", async function () {
            (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(1);
            (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("r3zUhJWabAMMLT5n631r2wDh9RP3dN1bRy");
            (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(342390045);
        });
        it("Should received amount ", async function () {
            (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(1);
            (0, chai_1.expect)(transaction.receivedAmounts[0].address).to.eq("rpE6gE8jEN1trDwQwe47VmgDL5y6m3XX2n");
            (0, chai_1.expect)(transaction.receivedAmounts[0].amount.toNumber()).to.eq(342390000);
        });
        it("Should get type ", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("Payment");
        });
        it("Should check if native payment ", async function () {
            (0, chai_1.expect)(transaction.isNativePayment).to.eq(true);
        });
        it("Should get currency name ", async function () {
            (0, chai_1.expect)(transaction.currencyName).to.eq("XRP");
        });
        it("Should get elementary unit ", async function () {
            (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(1000000);
        });
        it("Should get success status ", async function () {
            (0, chai_1.expect)(transaction.successStatus).to.eq(0);
        });
        it("Should get payment summary ", async function () {
            var _a, _b, _c, _d;
            const summary = await transaction.paymentSummary(MccClient);
            (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
            (0, chai_1.expect)(summary.sourceAddress).to.eq("r3zUhJWabAMMLT5n631r2wDh9RP3dN1bRy");
            (0, chai_1.expect)(summary.receivingAddress).to.eq("rpE6gE8jEN1trDwQwe47VmgDL5y6m3XX2n");
            (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(342390045);
            (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(342390000);
            (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
            (0, chai_1.expect)((_c = summary.tokenElementaryUnits) === null || _c === void 0 ? void 0 : _c.toNumber()).to.eq(undefined);
            (0, chai_1.expect)((_d = summary.receivedTokenAmount) === null || _d === void 0 ? void 0 : _d.toNumber()).to.eq(undefined);
            (0, chai_1.expect)(summary.oneToOne).to.eq(true);
            (0, chai_1.expect)(summary.isFull).to.eq(true);
        });
        it.skip("Should get payment summary ", async function () {
            const summary = await transaction.paymentSummary(MccClient);
            console.log(summary);
            if (summary.tokenElementaryUnits)
                console.log("tokenElementaryUnits: ", summary.tokenElementaryUnits.toString(10));
            if (summary.receivedTokenAmount)
                console.log("receivedTokenAmount: ", summary.receivedTokenAmount.toString(10));
            if (summary.spentAmount)
                console.log("spentAmount: ", summary.spentAmount.toString(10));
            if (summary.receivedAmount)
                console.log("receivedAmount: ", summary.receivedAmount.toString(10));
        });
    });
    describe("Transaction with no reference token transfer ", function () {
        let transaction;
        const txid = "84464F5001B9E7FD79C448B9C5F01085ACE56E94A1F6E2A737FDE9A993086F16";
        before(async function () {
            transaction = await MccClient.getTransaction(txid);
        });
        it("Should find transaction ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            (0, chai_1.expect)(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            (0, chai_1.expect)(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            (0, chai_1.expect)(transaction.hash).to.eq(txid);
        });
        it("Should get transaction reference array ", async function () {
            (0, chai_1.expect)(transaction.reference.length).to.eq(0);
        });
        it("Should get standardized transaction reference ", async function () {
            (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1646141781);
        });
        it("Should get source address ", async function () {
            (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("rP6JLXtRNs3tjeYnn7zUHpbfLjuyBXqhwF");
        });
        it("Should get receiving address ", async function () {
            (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(1);
            (0, chai_1.expect)(transaction.receivingAddresses[0]).to.eq("rBPCqK87DsSSZKewDV7QzASCysUUJA8abf");
        });
        it("Should get fee ", async function () {
            (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(10);
        });
        it("Should spend amount ", async function () {
            (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(1);
            (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("rP6JLXtRNs3tjeYnn7zUHpbfLjuyBXqhwF");
            (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(10);
        });
        it("Should received amount ", async function () {
            (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(0);
        });
        it("Should get type ", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("Payment");
        });
        it("Should check if native payment ", async function () {
            (0, chai_1.expect)(transaction.isNativePayment).to.eq(false);
        });
        it("Should get currency name ", async function () {
            (0, chai_1.expect)(transaction.currencyName).to.eq("426C657373656400000000000000000000000000");
        });
        it("Should get elementary unit ", async function () {
            (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(1000000);
        });
        it("Should get success status ", async function () {
            (0, chai_1.expect)(transaction.successStatus).to.eq(0);
        });
        it("Should get payment summary ", async function () {
            var _a, _b, _c, _d;
            const summary = await transaction.paymentSummary(MccClient);
            (0, chai_1.expect)(summary.isNativePayment).to.eq(false);
            (0, chai_1.expect)(summary.sourceAddress).to.eq("rP6JLXtRNs3tjeYnn7zUHpbfLjuyBXqhwF");
            (0, chai_1.expect)(summary.receivingAddress).to.eq("rBPCqK87DsSSZKewDV7QzASCysUUJA8abf");
            (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(10);
            (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(undefined);
            (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
            (0, chai_1.expect)((_c = summary.tokenElementaryUnits) === null || _c === void 0 ? void 0 : _c.toNumber()).to.eq(1);
            (0, chai_1.expect)((_d = summary.receivedTokenAmount) === null || _d === void 0 ? void 0 : _d.toNumber()).to.eq(400000);
            (0, chai_1.expect)(summary.oneToOne).to.eq(true);
            (0, chai_1.expect)(summary.isFull).to.eq(true);
        });
    });
});
//# sourceMappingURL=transaction.test.js.map