"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../../src");
const chai_1 = require("chai");
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
};
describe("Test utils ", function () {
    it("should convert classic account to bytes ", async function () {
        const acc = "r4BhzWSGGjTeSdpcXMPoT1AbiCQm76FQGd";
        const byts = (0, src_1.rippleAddressToBytes)(acc);
        (0, chai_1.expect)(byts.length).to.eq(20);
    });
    it("should convert x-account to bytes ", async function () {
        const acc = "XVLhHMPHU98es4dbozjVtdWzVrDjtV18pX8yuPT7y4xaEHi";
        const byts = (0, src_1.rippleAddressToBytes)(acc);
        (0, chai_1.expect)(byts.length).to.eq(20);
    });
    it("should check that classic and x-account to bytes match ", async function () {
        const acc = "XVLhHMPHU98es4dbozjVtdWzVrDjtV18pX8yuPT7y4xaEHi";
        const classicAcc = "rGWrZyQqhTp9Xu7G5Pkayo7bXjH4k4QYpf";
        const byts = (0, src_1.rippleAddressToBytes)(acc);
        const cByts = (0, src_1.rippleAddressToBytes)(classicAcc);
        (0, chai_1.expect)(byts.length).to.eq(20);
        (0, chai_1.expect)(cByts.length).to.eq(20);
        (0, chai_1.expect)(Buffer.compare(byts, cByts)).to.eq(0);
    });
    it("should encode and decode to same acc string ", async function () {
        const classicAcc = "rGWrZyQqhTp9Xu7G5Pkayo7bXjH4k4QYpf";
        const cByts = (0, src_1.rippleAddressToBytes)(classicAcc);
        const decode = (0, src_1.bytesToRippleAddress)(cByts);
        (0, chai_1.expect)(cByts.length).to.eq(20);
        (0, chai_1.expect)(classicAcc).to.eq(decode);
    });
    it("should decode from x-account to bytes and back to its classsic account ", async function () {
        const acc = "XVLhHMPHU98es4dbozjVtdWzVrDjtV18pX8yuPT7y4xaEHi";
        const classicAcc = "rGWrZyQqhTp9Xu7G5Pkayo7bXjH4k4QYpf";
        const byts = (0, src_1.rippleAddressToBytes)(acc);
        const decode = (0, src_1.bytesToRippleAddress)(byts);
        (0, chai_1.expect)(byts.length).to.eq(20);
        (0, chai_1.expect)(classicAcc).to.eq(decode);
    });
});
//# sourceMappingURL=utils.test.js.map