"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const strackTrace_1 = require("../src/utils/strackTrace");
const trace_1 = require("../src/utils/trace");
const managedTest_1 = require("./managedTest");
const chai = require('chai');
const expect = chai.expect;
chai.use(require('chai-as-promised'));
describe("Managed test", () => {
    let dec;
    before(async function () {
        trace_1.traceManager.displayRuntimeTrace = false;
        trace_1.traceManager.displayStateOnException = false;
        dec = new managedTest_1.ManagedTest();
    });
    beforeEach(async function () {
        trace_1.traceManager.clearTrace();
    });
    afterEach(async function () {
        //traceManager.showTrace();
    });
    after(async function () {
    });
    it("Stack trace", async () => {
        const stack = new strackTrace_1.StackTrace();
        expect(stack.stack).to.eq(`new StackTrace\nContext.<anonymous>\ncallFn\nTest.Runnable.run\nRunner.runTest\nnext\nnext\ncbHookRun\ndone\n`);
    });
    it("Stack trace find", async () => {
        const stack = new strackTrace_1.StackTrace();
        expect(stack.find(`new StackTrace`)).to.not.eq(null);
    });
    it("Managed method sync", async () => {
        dec.method(5, 6);
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.method(5,6)=36`);
    });
    it("Managed method async", async () => {
        await dec.asyncMethod(7, 8);
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.asyncMethod(7,8)=224`);
    });
    it("Managed getter", async () => {
        dec.getter;
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.getter()=1`);
    });
    it("Managed setter", async () => {
        dec.setter = 15;
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.setter(15)`);
    });
    it("Managed throw sync", async () => {
        expect(function () { dec.methodThrow(5, 0); }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.methodThrow(5,0)`);
    });
    it("Managed throw sync mcc error", async () => {
        expect(function () { dec.methodThrowMccError(); }).to.throw("InvalidData");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.methodThrowMccError()`);
    });
    it("Managed throw async", async () => {
        await expect(dec.asyncMethodThrow(7, 0)).to.be.rejectedWith("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.asyncMethodThrow(7,0)`);
    });
    it("Managed throw getter", async () => {
        expect(function () {
            dec.getterThrow;
        }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.getterThrow()`);
    });
    it("Managed throw setter", async () => {
        expect(function () { dec.setterThrow = 15; }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.setterThrow(15)`);
    });
    it("Managed function", async () => {
        (0, managedTest_1.TestFunctionCall)(1, "A");
        //traceManager.showTrace(true,false,true);
        //traceManager.showMethods();
        expect(trace_1.traceManager.firstTrace).to.eq(`.TestFunction(1,A)`);
    });
    it("Managed nested test", async () => {
        //traceManager.displayRuntimeTrace=true;
        dec.f1("123");
        //traceManager.showTrace(true,false,true);
        //traceManager.showMethods();
        expect(trace_1.traceManager.getAsync(0).trace.length).to.eq(10);
    });
    // it("Managed nested async test", async () => {
    //    //traceManager.displayRuntimeTrace=true;
    //    await dec.asyncNestedMethod();
    //    //traceManager.showTrace(true,false,true);
    //    //traceManager.showMethods();
    //    //expect(traceManager.firstTrace).to.eq(`ManagedTest.f1(123)`);
    // });
    // it("Managed nested async await test", async () => {
    //    //traceManager.displayRuntimeTrace=true;
    //    await dec.asyncNestedMethodAwait();
    //    //traceManager.showTrace(true,false,true);
    //    //traceManager.showMethods();
    //    //expect(traceManager.firstTrace).to.eq(`ManagedTest.f1(123)`);
    // });
    // it("Managed nested async test wait all on end", async () => {
    //    //traceManager.displayRuntimeTrace=true;
    //    await dec.asyncNestedMethodWaitOnEnd();
    //    //traceManager.showTrace(true,false,true);
    //    //traceManager.showMethods();
    //    //expect(traceManager.firstTrace).to.eq(`ManagedTest.f1(123)`);
    // });
    // it( "neki" , async () => {
    //    let a = new Test2();
    //    console.log(`${a} - ${a.test2.constructor.name}`);
    // });
});
//# sourceMappingURL=managedTest.test.js.map