"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addressToBtyeAddress = exports.sendMinimalUTXOTransaction = exports.round_for_ltc = exports.expectThrow = void 0;
const chai_1 = require("chai");
const src_1 = require("../src");
const algoUtils_1 = require("../src/utils/algoUtils");
const expectThrow = async (method, errorMessage) => {
    let error = null;
    try {
        await method;
    }
    catch (err) {
        error = err;
    }
    if (typeof errorMessage === "object") {
        for (let prop in errorMessage) {
            (0, chai_1.expect)(error).to.haveOwnProperty(prop);
            (0, chai_1.expect)(error[prop]).to.eq(errorMessage[prop]);
        }
    }
    else {
        (0, chai_1.expect)(error).to.equal(errorMessage);
    }
};
exports.expectThrow = expectThrow;
function round_for_ltc(input) {
    return parseFloat(input.toFixed(8));
}
exports.round_for_ltc = round_for_ltc;
async function sendMinimalUTXOTransaction(RPC, fromWalletLabel, from, to) {
    let unspent = await RPC.listUnspentTransactions(fromWalletLabel);
    let vin = [];
    let vinaddresses = [];
    let unspentAmount = 0;
    for (let utx of unspent) {
        console.log("ALL utxo", utx);
        if (utx.address == from) {
            console.log(utx);
            vin.push({
                txid: utx.txid,
                vout: utx.vout,
            });
            vinaddresses.push(utx.address);
            unspentAmount += utx.amount;
        }
    }
    const gas = 1e-5;
    const test_amount = 1e-5;
    if (unspentAmount < gas + test_amount) {
        console.error("Not enough founds on sender wallet");
        throw Error("Insufficient founds on sender wallet");
    }
    let vout = [
        { address: to, amount: test_amount },
        { address: from, amount: round_for_ltc(unspentAmount - test_amount - gas) },
    ];
    let a = await RPC.createRawTransaction(fromWalletLabel, vin, vout);
    let signkeys = [];
    for (let add of vinaddresses) {
        signkeys.push(await RPC.getPrivateKey(fromWalletLabel, add));
    }
    let signedTx = await RPC.signRawTransaction(fromWalletLabel, a, signkeys);
    const txId = await RPC.sendRawTransactionInBlock(fromWalletLabel, signedTx.hex);
    return txId;
}
exports.sendMinimalUTXOTransaction = sendMinimalUTXOTransaction;
function addressToBtyeAddress(address) {
    const algoKeyPair = (0, algoUtils_1.addressToHex)(address);
    return (0, algoUtils_1.hexToBytes)((0, src_1.unPrefix0x)(algoKeyPair.publicKey) + (0, src_1.unPrefix0x)(algoKeyPair.checksum));
}
exports.addressToBtyeAddress = addressToBtyeAddress;
//# sourceMappingURL=testUtils.js.map